--[[ Candle Library by Ninwa 

	The Candle Library initially darkens your screen
	by drawing a screen-sized black rectangle with varying alpha-
	values. This alpha value is how "dark" it is. You can change
	this value by using Candle.setIntensity(n) where n is a number
	between 0 and 255. 
	
	After this, it draws light-sources by blending white circles
	with our darkness in a framebuffer (so nothing else accidentally
	gets blended) and then draws it back to our original screen.
	
	To use the library you should first set your shroud intensity, and
	then define new lightsources by using:
	
		Candle.new(x,y,max_shimmer,min_shimmer)
		
	This will add a new 'Flame' to our candle, or in otherwords, a new
	lightsource. Candle will keep track of all of your current light-sources.
	All you have to do is make sure to call:
	
		Candle.update(dt)
		&
		Candle.draw()
	
	in the appropriate places.
	
	This library is in early alpha and someday I hope that it will incorporate
	more advanced lighting (directional, walls/shadows, etc.)
	
	-Ninwa
	
	PS: Special thanks to Kikito for MiddleClass.lua, an OO library for Lua.

]]--

require "MiddleClass.lua"

Candle = class('Candle')
Flame  = class('Flame')

-- The Candle library is going to take the liberty
-- of keeping track of all of your current Flames.
-- I need to do this because of how I'm doing the drawing.
-- At least for now until I refactor/rethink it.
Candle.flames = {}

function Candle.new(x,y,max_shimmer,min_shimmer)
	table.insert(Candle.flames, Flame:new(x,y,max_shimmer,min_shimmer))
end

-- Remove all existing flames and
-- reset our framebuffer (not sure if that
-- last part is neccessary.)
function Candle.clearFlames()
	Candle.flames = {}
	Candle.buffer = love.graphics.newFramebuffer(1024,1024)
end

function Candle.update(dt)
	for _, v in pairs(Candle.flames) do
		v:update(dt)
	end
end

-- We need to use a frame buffer so we can do
-- independent blending of our light sources and
-- anything else being drawn. One is good
-- our shroud without having to worry about
-- for all of our Flames though!
Candle.buffer = love.graphics.newFramebuffer(1024,1024)

function Candle.draw(translatex, translatey)
	if translatex == nil then translatex = 0 end
	if translatey == nil then translatey = 0 end
	
	Candle.buffer:renderTo(function() 	
		-- Shroud screen in darkness before lighting.
		love.graphics.setColor(0,0,0,Candle.shroud_intensity)
		
		love.graphics.rectangle('fill',0,0,1024,1024)
		for _, v in ipairs(Candle.flames) do
			-- Additive here so our lightsources blend properly!
			love.graphics.setBlendMode("additive")
			love.graphics.setColor(255,255,255,light_intensity)
			--love.graphics.circle('fill',v.x-translatex,v.y-translatey,v.shimmer_radius,256) 
			love.graphics.circle('fill',v.x-translatex,v.y-translatey,v.shimmer_radius,256) 
		end
	end)
	
	-- BUT WILL IT BLEND!?
	love.graphics.setBlendMode("multiplicative")
	love.graphics.draw(Candle.buffer,0,0)
	love.graphics.setBlendMode("alpha")
end
   
Candle.shrouds = {}
Candle.shroud_intensity = 150 

-- Sets world darkness, accepts numbers between
-- 0 and 255.
function Candle.setShroudIntensity(intensity)
	if intensity < 0 or 
	   intensity > 255 then
		return
	end
	self.shroud_intensity = intensity
end

function Flame:initialize(x,y,max_shimmer,min_shimmer)
	self.x = x
	self.y = y
	if max_shimmer == nil then self.max_shimmer = 50 
	else self.max_shimmer = max_shimmer end
	if min_shimmer == nil then self.min_shimmer = 50
	else self.min_shimmer = min_shimmer end
	
	self.light_intensity = 255
	self.shimmer_radius = self.max_shimmer-1
	self.shroud_intensity = 200
	self.magic = 1 -- Every code needs a little bit of magic.
end

function Flame:update(dt)
	-- "Shimmer" effect by growing and
	-- shrinking Candle radius
	
	if self.min_shimmer ~= self.max_shimmer then
		self.shimmer_radius  = self.shimmer_radius - 50 * dt * self.magic
		self.shimimer_radius = math.floor(self.shimmer_radius)
		if self.shimmer_radius >= self.max_shimmer then
			self.magic = self.magic * -1
		end

		if self.shimmer_radius <= self.min_shimmer then
			self.magic = self.magic * -1
		end
	end
end